@@ -6,7 +6,7 @@
 
 #include "core/or/or.h"
 #include "feature/client/addressmap.h"
-#include "lib/container/buffers.h"
+#include "lib/buf/buffers.h"
 #include "core/mainloop/connection.h"
 #include "feature/control/control.h"
 #include "app/config/config.h"
@@ -545,7 +545,6 @@ parse_socks5_client_request(const uint8_t *raw_data, socks_request_t *req,
   if (parsed == -1) {
     log_warn(LD_APP, "socks5: parsing failed - invalid client request");
     res = SOCKS_RESULT_INVALID;
-    socks_request_set_socks5_error(req, SOCKS5_GENERAL_ERROR);
     goto end;
   } else if (parsed == -2) {
     res = SOCKS_RESULT_TRUNCATED;
@@ -557,7 +556,6 @@ parse_socks5_client_request(const uint8_t *raw_data, socks_request_t *req,
 
   if (socks5_client_request_get_version(trunnel_req) != 5) {
     res = SOCKS_RESULT_INVALID;
-    socks_request_set_socks5_error(req, SOCKS5_GENERAL_ERROR);
     goto end;
   }
 
@@ -592,7 +590,6 @@ parse_socks5_client_request(const uint8_t *raw_data, socks_request_t *req,
       tor_addr_to_str(req->address, &destaddr, sizeof(req->address), 1);
     } break;
     default: {
-      socks_request_set_socks5_error(req, SOCKS5_ADDRESS_TYPE_NOT_SUPPORTED);
       res = -1;
     } break;
   }
@@ -773,10 +770,8 @@ handle_socks_message(const uint8_t *raw_data, size_t datalen,
     } else {
       res = parse_socks5_client_request(raw_data, req,
                                         datalen, drain_out);
-      if (BUG(res == SOCKS_RESULT_INVALID && req->replylen == 0)) {
-        socks_request_set_socks5_error(req, SOCKS5_GENERAL_ERROR);
-      }
       if (res != SOCKS_RESULT_DONE) {
+        socks_request_set_socks5_error(req, SOCKS5_GENERAL_ERROR);
         goto end;
       }
 
@@ -861,7 +856,7 @@ fetch_from_buf_socks(buf_t *buf, socks_request_t *req,
       case SOCKS_RESULT_TRUNCATED:
         if (datalen == n_pullup)
           return 0;
-        FALLTHROUGH;
+        /* FALLTHRU */
       case SOCKS_RESULT_MORE_EXPECTED:
         res = 0;
         break;
@@ -967,7 +962,7 @@ parse_socks(const char *data, size_t datalen, socks_request_t *req,
       strlcpy((char*)req->reply, SOCKS_PROXY_IS_NOT_AN_HTTP_PROXY_MSG,
               MAX_SOCKS_REPLY_LEN);
       req->replylen = strlen((char*)req->reply)+1;
-      FALLTHROUGH;
+      /* fall through */
     default: /* version is not socks4 or socks5 */
       log_warn(LD_APP,
                "Socks version %d not recognized. (This port is not an "
@@ -1072,10 +1067,7 @@ parse_socks_client(const uint8_t *data, size_t datalen,
           log_info(LD_NET, "SOCKS 5 client: need authentication.");
           *drain_out = -1;
           return 2;
-        default:
-          /* This wasn't supposed to be exhaustive; there are other
-           * authentication methods too. */
-          ;
+        /* fall through */
       }
 
       *reason = tor_strdup("server doesn't support any of our available "
