@@ -359,10 +359,9 @@ mock_networkstatus_get_latest_consensus(void)
 }
 
 static networkstatus_t *
-mock_networkstatus_get_reasonably_live_consensus(time_t now, int flavor)
+mock_networkstatus_get_live_consensus(time_t now)
 {
   (void) now;
-  (void) flavor;
 
   tt_assert(mock_ns);
 
@@ -382,8 +381,6 @@ test_responsible_hsdirs(void *arg)
 
   MOCK(networkstatus_get_latest_consensus,
        mock_networkstatus_get_latest_consensus);
-  MOCK(networkstatus_get_reasonably_live_consensus,
-       mock_networkstatus_get_reasonably_live_consensus);
 
   ns = networkstatus_get_latest_consensus();
 
@@ -420,8 +417,6 @@ test_responsible_hsdirs(void *arg)
   smartlist_clear(ns->routerstatus_list);
   networkstatus_vote_free(mock_ns);
   cleanup_nodelist();
-
-  UNMOCK(networkstatus_get_reasonably_live_consensus);
 }
 
 static void
@@ -471,8 +466,6 @@ test_desc_reupload_logic(void *arg)
 
   hs_init();
 
-  MOCK(networkstatus_get_reasonably_live_consensus,
-       mock_networkstatus_get_reasonably_live_consensus);
   MOCK(router_have_minimum_dir_info,
        mock_router_have_minimum_dir_info);
   MOCK(get_or_state,
@@ -637,7 +630,7 @@ test_disaster_srv(void *arg)
   get_disaster_srv(1, srv_one);
   get_disaster_srv(2, srv_two);
 
-  /* Check that the cached ones where updated */
+  /* Check that the cached ones were updated */
   tt_mem_op(cached_disaster_srv_one, OP_EQ, srv_one, DIGEST256_LEN);
   tt_mem_op(cached_disaster_srv_two, OP_EQ, srv_two, DIGEST256_LEN);
 
@@ -884,11 +877,9 @@ static smartlist_t *service_responsible_hsdirs = NULL;
 static smartlist_t *client_responsible_hsdirs = NULL;
 
 static networkstatus_t *
-mock_networkstatus_get_reasonably_live_consensus_service(time_t now,
-                                                         int flavor)
+mock_networkstatus_get_live_consensus_service(time_t now)
 {
   (void) now;
-  (void) flavor;
 
   if (mock_service_ns) {
     return mock_service_ns;
@@ -904,14 +895,13 @@ mock_networkstatus_get_reasonably_live_consensus_service(time_t now,
 static networkstatus_t *
 mock_networkstatus_get_latest_consensus_service(void)
 {
-  return mock_networkstatus_get_reasonably_live_consensus_service(0, 0);
+  return mock_networkstatus_get_live_consensus_service(0);
 }
 
 static networkstatus_t *
-mock_networkstatus_get_reasonably_live_consensus_client(time_t now, int flavor)
+mock_networkstatus_get_live_consensus_client(time_t now)
 {
   (void) now;
-  (void) flavor;
 
   if (mock_client_ns) {
     return mock_client_ns;
@@ -927,7 +917,7 @@ mock_networkstatus_get_reasonably_live_consensus_client(time_t now, int flavor)
 static networkstatus_t *
 mock_networkstatus_get_latest_consensus_client(void)
 {
-  return mock_networkstatus_get_reasonably_live_consensus_client(0, 0);
+  return mock_networkstatus_get_live_consensus_client(0);
 }
 
 /* Mock function because we are not trying to test the close circuit that does
@@ -1387,8 +1377,8 @@ run_reachability_scenario(const reachability_cfg_t *cfg, int num_scenario)
    * === Client setup ===
    */
 
-  MOCK(networkstatus_get_reasonably_live_consensus,
-       mock_networkstatus_get_reasonably_live_consensus_client);
+  MOCK(networkstatus_get_live_consensus,
+       mock_networkstatus_get_live_consensus_client);
   MOCK(networkstatus_get_latest_consensus,
        mock_networkstatus_get_latest_consensus_client);
 
@@ -1412,14 +1402,14 @@ run_reachability_scenario(const reachability_cfg_t *cfg, int num_scenario)
   tt_int_op(smartlist_len(client_responsible_hsdirs), OP_EQ, 6);
 
   UNMOCK(networkstatus_get_latest_consensus);
-  UNMOCK(networkstatus_get_reasonably_live_consensus);
+  UNMOCK(networkstatus_get_live_consensus);
 
   /*
    * === Service setup ===
    */
 
-  MOCK(networkstatus_get_reasonably_live_consensus,
-       mock_networkstatus_get_reasonably_live_consensus_service);
+  MOCK(networkstatus_get_live_consensus,
+       mock_networkstatus_get_live_consensus_service);
   MOCK(networkstatus_get_latest_consensus,
        mock_networkstatus_get_latest_consensus_service);
 
@@ -1446,7 +1436,7 @@ run_reachability_scenario(const reachability_cfg_t *cfg, int num_scenario)
   tt_int_op(smartlist_len(service_responsible_hsdirs), OP_EQ, 8);
 
   UNMOCK(networkstatus_get_latest_consensus);
-  UNMOCK(networkstatus_get_reasonably_live_consensus);
+  UNMOCK(networkstatus_get_live_consensus);
 
   /* Some testing of the values we just got from the client and service. */
   tt_mem_op(&client_blinded_pk, OP_EQ, &service_blinded_pk,
@@ -1697,8 +1687,8 @@ test_client_service_hsdir_set_sync(void *arg)
 
   MOCK(networkstatus_get_latest_consensus,
        mock_networkstatus_get_latest_consensus);
-  MOCK(networkstatus_get_reasonably_live_consensus,
-       mock_networkstatus_get_reasonably_live_consensus);
+  MOCK(networkstatus_get_live_consensus,
+       mock_networkstatus_get_live_consensus);
   MOCK(get_or_state,
        get_or_state_replacement);
   MOCK(hs_desc_encode_descriptor,
